package repository

import (
	"context"

	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/entity"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/product"
	"go.mongodb.org/mongo-driver/mongo"
)

// TagRepo 定義與 tag (Tags) 資料表相關的 CRUD 與查詢操作
type TagRepo interface {
	Base
	TagBindingRepo
	Index
}

type Base interface {
	// Create 新增一筆 Tag 資料
	Create(ctx context.Context, tag *entity.Tags) error
	// GetByID 根據 tag 的內部 ID 取得資料
	GetByID(ctx context.Context, id string) (*entity.Tags, error)
	// Update 更新現有的 Tag 資料
	Update(ctx context.Context, id string, tag TagModifyParams) error
	// Delete 刪除指定 ID 的 Tag 資料
	Delete(ctx context.Context, id string) error
	// List 根據查詢條件取得 Tag 資料列表
	// 回傳值分別為資料列表與符合條件的總筆數
	List(ctx context.Context, params TagQueryParams) ([]*entity.Tags, int64, error)
	// GetByIDs 根據查詢條件取得 Tag 資料列表
	GetByIDs(ctx context.Context, ids []string) ([]*entity.Tags, error)
}

// TagQueryParams 為查詢 Tags 時的參數結構
type TagQueryParams struct {
	Types    *product.ItemType // 過濾 Tag 類型
	Name     *string           // 過濾名稱（部分比對）
	ShowType *product.ShowType // 過濾顯示筐
	// 可根據需求增加其他查詢條件，如分頁、排序等
	PageSize  int64
	PageIndex int64
}

type TagModifyParams struct {
	Types    *product.ItemType // 過濾 Tag 類型
	Name     *string           // 過濾名稱（部分比對）
	ShowType *product.ShowType // 過濾顯示筐
	Cover    *string           `bson:"cover,omitempty"` // 封面圖片
}

// TagBindingRepo 定義與 tag binding (TagsBindingTable) 資料表相關的操作
type TagBindingRepo interface {
	// BindTags 建立一筆 tag 與其他資料（例如專案）的綁定關係
	BindTags(ctx context.Context, binding []*entity.TagsBindingTable) error
	// UnbindTag 刪除一筆綁定資料
	UnbindTag(ctx context.Context, tagID, referenceID string) error
	// UnbindTagByReferenceID 刪除一筆綁定資料
	UnbindTagByReferenceID(ctx context.Context, referenceID string) error
	// GetBindingsByReference 根據參照 ID 取得所有綁定資料
	GetBindingsByReference(ctx context.Context, referenceID string) ([]*entity.TagsBindingTable, error)
	// ListTagBinding 根據查詢條件取得 tag binding 的資料列表
	// 回傳值分別為資料列表與符合條件的總筆數
	ListTagBinding(ctx context.Context, params TagBindingQueryParams) ([]*entity.TagsBindingTable, int64, error)
}

// TagBindingQueryParams 為查詢 TagBindingTable 時的參數結構
type TagBindingQueryParams struct {
	ReferenceID *string // 過濾參照 ID
	TagID       *string // 過濾 Tag ID
	// 可根據需求增加其他查詢條件，如分頁、排序等
	PageSize  int64
	PageIndex int64
}

type Index interface {
	IndexTags20250317001UP(ctx context.Context) (*mongo.Cursor, error)
	IndexTagsBinding20250317001UP(ctx context.Context) (*mongo.Cursor, error)
}
