package usecase

import (
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/entity"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/product"
	repo "code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/repository"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/usecase"
	mockRepository "code.30cm.net/digimon/app-cloudep-product-service/pkg/mock/repository"
	"context"
	"errors"
	"github.com/stretchr/testify/assert"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.uber.org/mock/gomock"
	"testing"
)

func TestTagsUseCase_Create(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)

	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	input := &entity.Tags{
		Name:     "Test Tag",
		Types:    product.ItemTypeProduct,
		ShowType: product.ShowTypeNormal,
	}

	t.Run("建立成功", func(t *testing.T) {
		mockTagsRepo.EXPECT().Create(ctx, input).Return(nil)

		err := useCase.Create(ctx, input)
		assert.NoError(t, err)
	})

	t.Run("建立失敗 - DB 錯誤", func(t *testing.T) {
		mockTagsRepo.EXPECT().Create(ctx, input).Return(errors.New("db error"))

		err := useCase.Create(ctx, input)
		assert.Error(t, err)
		assert.Contains(t, err.Error(), "failed to create tags")
	})
}

func TestTagsUseCase_GetByID(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)

	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	id := primitive.NewObjectID().Hex()

	t.Run("查詢成功", func(t *testing.T) {
		expected := &entity.Tags{
			ID:       primitive.NewObjectID(),
			Name:     "Test Tag",
			Types:    product.ItemTypeProduct,
			ShowType: product.ShowTypeNormal,
		}
		mockTagsRepo.EXPECT().GetByID(ctx, id).Return(expected, nil)

		tag, err := useCase.GetByID(ctx, id)
		assert.NoError(t, err)
		assert.Equal(t, expected, tag)
	})

	t.Run("查詢失敗 - DB 錯誤", func(t *testing.T) {
		mockTagsRepo.EXPECT().GetByID(ctx, id).Return(nil, errors.New("db error"))

		tag, err := useCase.GetByID(ctx, id)
		assert.Error(t, err)
		assert.Nil(t, tag)
		assert.Contains(t, err.Error(), "failed to get tags")
	})
}

func TestTagsUseCase_GetByIDs(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)

	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	ids := []string{"id1", "id2"}

	t.Run("查詢成功", func(t *testing.T) {
		expected := []*entity.Tags{
			{Name: "Tag1", Types: product.ItemTypeProduct, ShowType: product.ShowTypeNormal},
			{Name: "Tag2", Types: product.ItemTypeProduct, ShowType: product.ShowTypeNormal},
		}
		mockTagsRepo.EXPECT().GetByIDs(ctx, ids).Return(expected, nil)

		tags, err := useCase.GetByIDs(ctx, ids)
		assert.NoError(t, err)
		assert.Equal(t, expected, tags)
	})

	t.Run("查詢失敗 - DB 錯誤", func(t *testing.T) {
		mockTagsRepo.EXPECT().GetByIDs(ctx, ids).Return(nil, errors.New("db error"))

		tags, err := useCase.GetByIDs(ctx, ids)
		assert.Error(t, err)
		assert.Nil(t, tags)
		assert.Contains(t, err.Error(), "failed to get tags")
	})
}

func TestTagsUseCase_Delete(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)

	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	id := primitive.NewObjectID().Hex()

	t.Run("刪除成功", func(t *testing.T) {
		mockTagsRepo.EXPECT().Delete(ctx, id).Return(nil)
		err := useCase.Delete(ctx, id)
		assert.NoError(t, err)
	})

	t.Run("刪除失敗 - DB 錯誤", func(t *testing.T) {
		mockTagsRepo.EXPECT().Delete(ctx, id).Return(errors.New("db error"))
		err := useCase.Delete(ctx, id)
		assert.Error(t, err)
		assert.Contains(t, err.Error(), "failed to delete tags")
	})
}

func TestTagsUseCase_List(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)
	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	params := usecase.TagQueryParams{
		PageSize:  10,
		PageIndex: 1,
		Types:     ptr(product.ItemTypeProduct),
		Name:      ptr("AI"),
		ShowType:  ptr(product.ShowTypeNormal),
	}

	t.Run("查詢成功", func(t *testing.T) {
		expected := []*entity.Tags{
			{Name: "AI", Types: product.ItemTypeProduct, ShowType: product.ShowTypeNormal},
		}
		mockTagsRepo.EXPECT().List(ctx, repo.TagQueryParams{
			PageSize:  10,
			PageIndex: 1,
			Types:     ptr(product.ItemTypeProduct),
			Name:      ptr("AI"),
			ShowType:  ptr(product.ShowTypeNormal),
		}).Return(expected, int64(1), nil)

		res, total, err := useCase.List(ctx, params)
		assert.NoError(t, err)
		assert.Equal(t, expected, res)
		assert.Equal(t, int64(1), total)
	})

	t.Run("查詢失敗 - DB 錯誤", func(t *testing.T) {
		dbErr := errors.New("db error")
		mockTagsRepo.EXPECT().List(ctx, repo.TagQueryParams{
			PageSize:  10,
			PageIndex: 1,
			Types:     ptr(product.ItemTypeProduct),
			Name:      ptr("AI"),
			ShowType:  ptr(product.ShowTypeNormal),
		}).Return(nil, int64(0), dbErr)

		res, total, err := useCase.List(ctx, params)
		assert.Error(t, err)
		assert.Nil(t, res)
		assert.Equal(t, int64(0), total)
		assert.Contains(t, err.Error(), "failed to list tags")
	})
}

func TestTagsUseCase_Update(t *testing.T) {
	mockCtrl := gomock.NewController(t)
	defer mockCtrl.Finish()

	mockTagsRepo := mockRepository.NewMockTagRepo(mockCtrl)
	useCase := MustTagsUseCase(TagsUseCaseParam{
		TagsRepo: mockTagsRepo,
	})

	ctx := context.Background()
	id := primitive.NewObjectID().Hex()
	tagParams := usecase.TagModifyParams{
		Name:     ptr("New Name"),
		Types:    ptr(product.ItemTypeProduct),
		ShowType: ptr(product.ShowTypeNormal),
		Cover:    ptr("https://image.jpg"),
	}

	t.Run("更新成功", func(t *testing.T) {
		mockTagsRepo.EXPECT().Update(ctx, id, repo.TagModifyParams{
			Name:     tagParams.Name,
			Types:    tagParams.Types,
			ShowType: tagParams.ShowType,
			Cover:    tagParams.Cover,
		}).Return(nil)
		err := useCase.Update(ctx, id, tagParams)
		assert.NoError(t, err)
	})

	t.Run("更新失敗 - DB 錯誤", func(t *testing.T) {
		dbErr := errors.New("db error")
		mockTagsRepo.EXPECT().Update(ctx, id, repo.TagModifyParams{
			Name:     tagParams.Name,
			Types:    tagParams.Types,
			ShowType: tagParams.ShowType,
			Cover:    tagParams.Cover,
		}).Return(dbErr)
		err := useCase.Update(ctx, id, tagParams)
		assert.Error(t, err)
		assert.Contains(t, err.Error(), "failed to update tags")
	})
}
